/*
 *  Copyright (c) Members of the EGEE Collaboration. 2005.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  gLite Data Management - Simple SRM API
 *
 *  Authors: 
 *      Ricardo Rocha <Ricardo.Rocha@cern.ch>
 *      From Gabor Gombas code in the catalog-cli module
 *  Version info: $Id: srm-simple.h,v 1.7 2007/01/31 09:04:35 badino Exp $
 *  Release: $Name: glite-data-srm-cli_R_1_2_8_3 $
 *
 */

#ifndef GLITE_DATA_SRM_SIMPLE_H
#define GLITE_DATA_SRM_SIMPLE_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdarg.h>
#include <stdint.h>
#include <time.h>

#include <glite/data/srm/c/srmStub.h>

/**********************************************************************
 * Gsoap name wrappers
 */

#if _GSOAP_VERSION >= 0x020700
#define ArrayOfstring       srm4ArrayOfstring 
#define ArrayOfboolean      srm5ArrayOfboolean
#define ArrayOflong         srm5ArrayOflong
#define ArrayOfFileMetaData srm2ArrayOfFileMetaData 
#endif

	
/**********************************************************************
 * Data type declarations
 */

/*
 * Opaque data structure used by the library.
 */
typedef struct _glite_srm_ctx		glite_srm_ctx;

/**********************************************************************
 * Constants
 */

/**
 * \brief Error categories. These are analogous to the exception classes in Java.
 */
typedef enum
{
	GLITE_SRM_ERROR_NONE,
	GLITE_SRM_ERROR_UNKNOWN,
	GLITE_SRM_ERROR_OUTOFMEMORY,
	GLITE_SRM_ERROR_SERVICEDISCOVERY,
	GLITE_SRM_ERROR_SOAP
} glite_srm_errclass;

/**
 * \brief Service type used for Service Discovery.
 */
#define GLITE_SRM_SD_TYPE		"SRM"

/**
 * @}
 */

/**********************************************************************
 * General guidelines:
 * - Functions that return a pointer return NULL when there is an error.
 * - Functions that return 'int' return 0 when successful and -1 in case
 *   of an error.
 * - Any objects returned by a function is owned by the caller and has to
 *   be deallocated by the caller.
 */

/**
 * \brief Allocates a new srm context. 
 *
 * The context can be used to access only one service. 
 * 
 * If the endpoint is not a URL (HTTP, HTTPS or HTTPG), then
 * service discovery is used to locate the real service endpoint.
 * In this case the 'endpoint' parameter is the input of the 
 * service discovery. 
 *
 * This mechanism is executed upon the first service specific usage 
 * of the context. After that point methods using the same context,
 * from any other interface will return an error.
 *
 * @param endpoint The name of the endpoint. Either an URL or a
 *  service name.
 * @return the context or NULL if memory allocation has failed.
 */
glite_srm_ctx *glite_srm_new(const char *endpoint);

/**
 * Destroys a srm context.
 *
 * @param ctx The global context.
 */
void glite_srm_free(glite_srm_ctx *ctx);

/** 
 * Get the current endpoint.
 *
 * @param ctx The global context.
 * @return the current endpoint used by the srm context.
 */
const char *glite_srm_get_endpoint(glite_srm_ctx *ctx);

/**
 * \brief Gets the error message for the last failed operation.
 *
 * The returned pointer is valid only till the next call to any of
 * the library's functions with the same context pointer.
 *
 * @param ctx The global context.
 * @return the error message.
 */
const char *glite_srm_get_error(glite_srm_ctx *ctx);

/**
 * \brief Determines the class of the last error.
 *
 * @param ctx The global context.
 * @return the proper glite_srm_errclass.
 */
glite_srm_errclass glite_srm_get_errclass(glite_srm_ctx *ctx);

/**
 * \brief Set the error message in the context.
 *
 * @param ctx The global context.
 * @param errclass The error class.
 * @param fmt The format of the error message.
 */
void glite_srm_set_error(glite_srm_ctx *ctx,
	glite_srm_errclass errclass, const char *fmt, ...);

/**
 * \brief Set the error message in the context.
 * 
 * @param ctx The global context.
 * @param errclass The error class.
 * @param fmt The format of the error message.
 * @param ap The variable-length argument list. 
 */
void glite_srm_set_verror(glite_srm_ctx *ctx,
	glite_srm_errclass errclass, const char *fmt, va_list ap);

/**
 * @}
 */

/**
 * Interface Methods
 */

int glite_srm_ping(glite_srm_ctx *ctx);

char** glite_srm_get_protocols(glite_srm_ctx *ctx, int *resultCnt);

int glite_srm_delete(glite_srm_ctx *ctx, const char * surls[], int cnt);

struct srm2__RequestStatus *glite_srm_pin(glite_srm_ctx *ctx, const char * turls[], int cnt);

struct srm2__RequestStatus *glite_srm_unpin(glite_srm_ctx *ctx, int requestId, 
		const char * turls[], int cnt);

struct srm2__RequestStatus *glite_srm_get(glite_srm_ctx *ctx, const char * surls[], 
		const char * protocols[], int cnt, int estimated);

struct srm2__RequestStatus *glite_srm_put(glite_srm_ctx *ctx, const char * surls[], 
		const char * protocols[], int cnt);

struct srm2__RequestStatus *glite_srm_mkPermanent(glite_srm_ctx *ctx, const char * surls[], int cnt);

struct srm2__RequestStatus *glite_srm_get_status(glite_srm_ctx *ctx, int requestId);

struct srm2__RequestStatus *glite_srm_set_status(glite_srm_ctx *ctx, int requestId, 
		int fileId, char *state);

struct ArrayOfFileMetaData *glite_srm_getFileMetaData(glite_srm_ctx *ctx, const char * surls[], int cnt);


#ifdef __cplusplus
}
#endif

#endif /* GLITE_DATA_SRM_SIMPLE_H */
